package com.sun.jmx.snmp.agent;

import java.util.Arrays;
import java.util.Iterator;
import com.sun.jmx.snmp.SnmpEngine;
import com.sun.jmx.snmp.SnmpPdu;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import com.sun.jmx.snmp.SnmpOid;
import com.sun.jmx.snmp.SnmpVarBind;
import java.util.Enumeration;
import com.sun.jmx.snmp.SnmpStatusException;
import java.util.Hashtable;

final class SnmpRequestTree
{
    private Hashtable<Object, SnmpRequestTree.Handler> hashtable;
    private SnmpMibRequest request;
    private int version;
    private boolean creationflag;
    private boolean getnextflag;
    private int type;
    private boolean setreqflag;
    
    SnmpRequestTree(SnmpMibRequest request, boolean creationflag, int pduType) {
        this.hashtable = null;
        this.request = null;
        this.version = 0;
        this.creationflag = false;
        this.getnextflag = false;
        this.type = 0;
        this.setreqflag = false;
        this.request = request;
        this.version = request.getVersion();
        this.creationflag = creationflag;
        this.hashtable = new Hashtable();
        this.setPduType(pduType);
    }
    
    public static int mapSetException(int n, int n2) throws SnmpStatusException {
        if (n2 == 0) {
            return n;
        }
        if ((n = n) == 225) {
            n = 17;
        }
        else if (n == 224) {
            n = 17;
        }
        return n;
    }
    
    public static int mapGetException(int n, int n2) throws SnmpStatusException {
        if (n2 == 0) {
            return n;
        }
        if ((n = n) == 225) {
        }
        else if (n == 224) {
        }
        else if (n == 6) {
            n = 224;
        }
        else if (n == 18) {
            n = 224;
        }
        else if (n >= 7 && n <= 12) {
            n = 224;
        }
        else if (n == 4) {
            n = 224;
        }
        else if (n != 16 && n != 5) {
            n = 225;
        }
        return n;
    }
    
    public Object getUserData() {
        return this.request.getUserData();
    }
    
    public boolean isCreationAllowed() {
        return this.creationflag;
    }
    
    public boolean isSetRequest() {
        return this.setreqflag;
    }
    
    public int getVersion() {
        return this.version;
    }
    
    public int getRequestPduVersion() {
        return this.request.getRequestPduVersion();
    }
    
    public SnmpMibNode getMetaNode(SnmpRequestTree.Handler handler) {
        return handler.meta;
    }
    
    public int getOidDepth(SnmpRequestTree.Handler handler) {
        return handler.depth;
    }
    
    public Enumeration<SnmpMibSubRequest> getSubRequests(SnmpRequestTree.Handler handler) {
        return new SnmpRequestTree.Enum(this, handler);
    }
    
    public Enumeration<SnmpRequestTree.Handler> getHandlers() {
        return this.hashtable.elements();
    }
    
    public void add(SnmpMibNode snmpMibNode, int n, SnmpVarBind snmpVarBind) throws SnmpStatusException {
        this.registerNode(snmpMibNode, n, null, snmpVarBind, false, null);
    }
    
    public void add(SnmpMibNode snmpMibNode, int n, SnmpOid snmpOid, SnmpVarBind snmpVarBind, boolean b) throws SnmpStatusException {
        this.registerNode(snmpMibNode, n, snmpOid, snmpVarBind, b, null);
    }
    
    public void add(SnmpMibNode snmpMibNode, int n, SnmpOid snmpOid, SnmpVarBind snmpVarBind, boolean b, SnmpVarBind snmpVarBind2) throws SnmpStatusException {
        this.registerNode(snmpMibNode, n, snmpOid, snmpVarBind, b, snmpVarBind2);
    }
    
    void setPduType(int type) {
        this.type = type;
        this.setreqflag = (type == 253 || type == 163);
    }
    
    void setGetNextFlag() {
        this.getnextflag = true;
    }
    
    void switchCreationFlag(boolean creationflag) {
        this.creationflag = creationflag;
    }
    
    SnmpMibSubRequest getSubRequest(SnmpRequestTree.Handler handler) {
        if (handler == null) {
            return null;
        }
        return new SnmpRequestTree.SnmpMibSubRequestImpl(this.request, handler.getSubList(), null, false, this.getnextflag, null);
    }
    
    SnmpMibSubRequest getSubRequest(SnmpRequestTree.Handler handler, SnmpOid snmpOid) {
        if (handler == null) {
            return null;
        }
        int entryPos = handler.getEntryPos(snmpOid);
        if (entryPos == -1) {
            return null;
        }
        return new SnmpRequestTree.SnmpMibSubRequestImpl(this.request, handler.getEntrySubList(entryPos), handler.getEntryOid(entryPos), handler.isNewEntry(entryPos), this.getnextflag, handler.getRowStatusVarBind(entryPos));
    }
    
    SnmpMibSubRequest getSubRequest(SnmpRequestTree.Handler handler, int n) {
        if (handler == null) {
            return null;
        }
        return new SnmpRequestTree.SnmpMibSubRequestImpl(this.request, handler.getEntrySubList(n), handler.getEntryOid(n), handler.isNewEntry(n), this.getnextflag, handler.getRowStatusVarBind(n));
    }
    
    private void put(Object key, SnmpRequestTree.Handler value) {
        if (value == null) {
            return;
        }
        if (key == null) {
            return;
        }
        if (this.hashtable == null) {
            this.hashtable = new Hashtable();
        }
        this.hashtable.put(key, value);
        return;
    }
    
    private SnmpRequestTree.Handler get(Object key) {
        if (key == null) {
            return null;
        }
        if (this.hashtable == null) {
            return null;
        }
        return (SnmpRequestTree.Handler)this.hashtable.get(key);
    }
    
    private static int findOid(SnmpOid[] array, int n, SnmpOid snmpOid) {
        int i = 0;
        int n2 = n - 1;
        int n3 = i + (n2 - i) / 2;
        while (i <= n2) {
            SnmpOid snmpOid2 = array[n3];
            int compareTo = snmpOid.compareTo(snmpOid2);
            if (compareTo == 0) {
                return n3;
            }
            if (snmpOid.equals(snmpOid2)) {
                return n3;
            }
            if (compareTo > 0) {
                i = n3 + 1;
            }
            else {
                n2 = n3 - 1;
            }
            n3 = i + (n2 - i) / 2;
        }
        return -1;
    }
    
    private static int getInsertionPoint(SnmpOid[] array, int n, SnmpOid snmpOid) {
        int i = 0;
        int n2 = n - 1;
        int n3 = i + (n2 - i) / 2;
        while (i <= n2) {
            int compareTo = snmpOid.compareTo(array[n3]);
            if (compareTo == 0) {
                return n3;
            }
            if (compareTo > 0) {
                i = n3 + 1;
            }
            else {
                n2 = n3 - 1;
            }
            n3 = i + (n2 - i) / 2;
        }
        return n3;
    }
    
    private void registerNode(SnmpMibNode meta, int depth, SnmpOid snmpOid, SnmpVarBind snmpVarBind, boolean b, SnmpVarBind snmpVarBind2) throws SnmpStatusException {
        if (meta == null) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpRequestTree.class.getName(), "registerNode", "meta-node is null!");
            return;
        }
        if (snmpVarBind == null) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpRequestTree.class.getName(), "registerNode", "varbind is null!");
            return;
        }
        SnmpRequestTree.Handler value = this.get(meta);
        if (value == null) {
            value = new SnmpRequestTree.Handler(this.type);
            value.meta = meta;
            value.depth = depth;
            this.put(meta, value);
        }
        if (snmpOid == null) {
            value.addVarbind(snmpVarBind);
        }
        else {
            value.addVarbind(snmpVarBind, snmpOid, b, snmpVarBind2);
        }
        return;
    }
    
    static int access$000(SnmpOid[] array, int n, SnmpOid snmpOid) {
        return getInsertionPoint(array, n, snmpOid);
    }
    
    static int access$100(SnmpOid[] array, int n, SnmpOid snmpOid) {
        return findOid(array, n, snmpOid);
    }
    
    static final class Enum implements Enumeration<SnmpMibSubRequest>
    {
        private final SnmpRequestTree.Handler handler;
        private final SnmpRequestTree hlist;
        private int entry;
        private int iter;
        private int size;
        
        Enum(SnmpRequestTree hlist, SnmpRequestTree.Handler handler) {
            this.entry = 0;
            this.iter = 0;
            this.size = 0;
            this.handler = handler;
            this.hlist = hlist;
            this.size = handler.getSubReqCount();
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.iter < this.size;
        }
        
        @Override
        public SnmpMibSubRequest nextElement() throws NoSuchElementException {
            if (this.iter == 0 && this.handler.sublist != null) {
                ++this.iter;
                return this.hlist.getSubRequest(this.handler);
            }
            ++this.iter;
            if (this.iter > this.size) {
                throw new NoSuchElementException();
            }
            SnmpMibSubRequest subRequest = this.hlist.getSubRequest(this.handler, this.entry);
            ++this.entry;
            return subRequest;
        }
    }
    
    static final class SnmpMibSubRequestImpl implements SnmpMibSubRequest
    {
        private final Vector<SnmpVarBind> varbinds;
        private final SnmpMibRequest global;
        private final int version;
        private final boolean isnew;
        private final SnmpOid entryoid;
        private final boolean getnextflag;
        private final SnmpVarBind statusvb;
        
        SnmpMibSubRequestImpl(SnmpMibRequest global, Vector<SnmpVarBind> varbinds, SnmpOid entryoid, boolean isnew, boolean getnextflag, SnmpVarBind statusvb) {
            this.global = global;
            this.varbinds = varbinds;
            this.version = global.getVersion();
            this.entryoid = entryoid;
            this.isnew = isnew;
            this.getnextflag = getnextflag;
            this.statusvb = statusvb;
        }
        
        @Override
        public Enumeration<SnmpVarBind> getElements() {
            return this.varbinds.elements();
        }
        
        @Override
        public Vector<SnmpVarBind> getSubList() {
            return this.varbinds;
        }
        
        @Override
        public final int getSize() {
            if (this.varbinds == null) {
                return 0;
            }
            return this.varbinds.size();
        }
        
        @Override
        public void addVarBind(SnmpVarBind obj) {
            this.varbinds.addElement(obj);
            this.global.addVarBind(obj);
        }
        
        @Override
        public boolean isNewEntry() {
            return this.isnew;
        }
        
        @Override
        public SnmpOid getEntryOid() {
            return this.entryoid;
        }
        
        @Override
        public int getVarIndex(SnmpVarBind snmpVarBind) {
            if (snmpVarBind == null) {
                return 0;
            }
            return this.global.getVarIndex(snmpVarBind);
        }
        
        @Override
        public Object getUserData() {
            return this.global.getUserData();
        }
        
        @Override
        public void registerGetException(SnmpVarBind snmpVarBind, SnmpStatusException ex) throws SnmpStatusException {
            if (this.version == 0) {
                throw new SnmpStatusException(ex, this.getVarIndex(snmpVarBind) + 1);
            }
            if (snmpVarBind == null) {
                throw ex;
            }
            if (this.getnextflag) {
                snmpVarBind.value = SnmpVarBind.endOfMibView;
                return;
            }
            int mapGetException = SnmpRequestTree.mapGetException(ex.getStatus(), this.version);
            if (mapGetException == 225) {
                snmpVarBind.value = SnmpVarBind.noSuchObject;
            }
            else if (mapGetException == 224) {
                snmpVarBind.value = SnmpVarBind.noSuchInstance;
            }
            else {
                throw new SnmpStatusException(mapGetException, this.getVarIndex(snmpVarBind) + 1);
            }
            return;
        }
        
        @Override
        public void registerSetException(SnmpVarBind snmpVarBind, SnmpStatusException ex) throws SnmpStatusException {
            if (this.version == 0) {
                throw new SnmpStatusException(ex, this.getVarIndex(snmpVarBind) + 1);
            }
            throw new SnmpStatusException(15, this.getVarIndex(snmpVarBind) + 1);
        }
        
        @Override
        public void registerCheckException(SnmpVarBind snmpVarBind, SnmpStatusException ex) throws SnmpStatusException {
            int status = ex.getStatus();
            int mapSetException = SnmpRequestTree.mapSetException(status, this.version);
            if (status != mapSetException) {
                throw new SnmpStatusException(mapSetException, this.getVarIndex(snmpVarBind) + 1);
            }
            throw new SnmpStatusException(ex, this.getVarIndex(snmpVarBind) + 1);
        }
        
        @Override
        public int getVersion() {
            return this.version;
        }
        
        @Override
        public SnmpVarBind getRowStatusVarBind() {
            return this.statusvb;
        }
        
        @Override
        public SnmpPdu getPdu() {
            return this.global.getPdu();
        }
        
        @Override
        public int getRequestPduVersion() {
            return this.global.getRequestPduVersion();
        }
        
        @Override
        public SnmpEngine getEngine() {
            return this.global.getEngine();
        }
        
        @Override
        public String getPrincipal() {
            return this.global.getPrincipal();
        }
        
        @Override
        public int getSecurityLevel() {
            return this.global.getSecurityLevel();
        }
        
        @Override
        public int getSecurityModel() {
            return this.global.getSecurityModel();
        }
        
        @Override
        public byte[] getContextName() {
            return this.global.getContextName();
        }
        
        @Override
        public byte[] getAccessContextName() {
            return this.global.getAccessContextName();
        }
    }
    
    static final class Handler
    {
        SnmpMibNode meta;
        int depth;
        Vector<SnmpVarBind> sublist;
        SnmpOid[] entryoids;
        Vector<SnmpVarBind>[] entrylists;
        boolean[] isentrynew;
        SnmpVarBind[] rowstatus;
        int entrycount;
        int entrysize;
        final int type;
        private static final int Delta = 10;
        
        public Handler(int type) {
            this.entryoids = null;
            this.entrylists = null;
            this.isentrynew = null;
            this.rowstatus = null;
            this.entrycount = 0;
            this.entrysize = 0;
            this.type = type;
        }
        
        public void addVarbind(SnmpVarBind obj) {
            if (this.sublist == null) {
                this.sublist = new Vector();
            }
            this.sublist.addElement(obj);
        }
        
        void add(int entrycount, SnmpOid snmpOid, Vector<SnmpVarBind> vector, boolean b, SnmpVarBind snmpVarBind) {
            if (this.entryoids == null) {
                this.entryoids = new SnmpOid[10];
                this.entrylists = (Vector[])new Vector[10];
                this.isentrynew = new boolean[10];
                this.rowstatus = new SnmpVarBind[10];
                this.entrysize = 10;
                entrycount = 0;
            }
            else if (entrycount >= this.entrysize || this.entrycount == this.entrysize) {
                SnmpOid[] entryoids = this.entryoids;
                Vector[] entrylists = this.entrylists;
                boolean[] isentrynew = this.isentrynew;
                SnmpVarBind[] rowstatus = this.rowstatus;
                this.entrysize += 10;
                this.entryoids = new SnmpOid[this.entrysize];
                this.entrylists = (Vector[])new Vector[this.entrysize];
                this.isentrynew = new boolean[this.entrysize];
                this.rowstatus = new SnmpVarBind[this.entrysize];
                if (entrycount > this.entrycount) {
                    entrycount = this.entrycount;
                }
                if (entrycount < 0) {
                    entrycount = 0;
                }
                int n3 = this.entrycount - entrycount;
                if (entrycount > 0) {
                    System.arraycopy(entryoids, 0, this.entryoids, 0, entrycount);
                    System.arraycopy(entrylists, 0, this.entrylists, 0, entrycount);
                    System.arraycopy(isentrynew, 0, this.isentrynew, 0, entrycount);
                    System.arraycopy(rowstatus, 0, this.rowstatus, 0, entrycount);
                }
                if (n3 > 0) {
                    int n4 = entrycount + 1;
                    System.arraycopy(entryoids, entrycount, this.entryoids, n4, n3);
                    System.arraycopy(entrylists, entrycount, this.entrylists, n4, n3);
                    System.arraycopy(isentrynew, entrycount, this.isentrynew, n4, n3);
                    System.arraycopy(rowstatus, entrycount, this.rowstatus, n4, n3);
                }
            }
            else if (entrycount < this.entrycount) {
                int n = entrycount + 1;
                int n2 = this.entrycount - entrycount;
                System.arraycopy(this.entryoids, entrycount, this.entryoids, n, n2);
                System.arraycopy(this.entrylists, entrycount, this.entrylists, n, n2);
                System.arraycopy(this.isentrynew, entrycount, this.isentrynew, n, n2);
                System.arraycopy(this.rowstatus, entrycount, this.rowstatus, n, n2);
            }
            this.entryoids[entrycount] = snmpOid;
            this.entrylists[entrycount] = vector;
            this.isentrynew[entrycount] = b;
            this.rowstatus[entrycount] = snmpVarBind;
            ++this.entrycount;
        }
        
        public void addVarbind(SnmpVarBind obj, SnmpOid snmpOid, boolean b, SnmpVarBind snmpVarBind) throws SnmpStatusException {
            Vector<SnmpVarBind> vector;
            if (this.entryoids == null) {
                vector = new Vector();
                this.add(0, snmpOid, vector, b, snmpVarBind);
            }
            else {
                int access$000 = SnmpRequestTree.access$000(this.entryoids, this.entrycount, snmpOid);
                if (access$000 > -1 && access$000 < this.entrycount && snmpOid.compareTo(this.entryoids[access$000]) == 0) {
                    vector = this.entrylists[access$000];
                    snmpVarBind = this.rowstatus[access$000];
                }
                else {
                    vector = new Vector();
                    this.add(access$000, snmpOid, vector, b, snmpVarBind);
                }
                if (snmpVarBind != null) {
                    if (snmpVarBind != null && snmpVarBind != snmpVarBind && (this.type == 253 || this.type == 163)) {
                        throw new SnmpStatusException(12);
                    }
                    this.rowstatus[access$000] = snmpVarBind;
                }
            }
            if (snmpVarBind != obj) {
                vector.addElement(obj);
            }
        }
        
        public int getSubReqCount() {
            int n = 0;
            if (this.sublist != null) {
                ++n;
            }
            if (this.entryoids != null) {
                n += this.entrycount;
            }
            return n;
        }
        
        public Vector<SnmpVarBind> getSubList() {
            return this.sublist;
        }
        
        public int getEntryPos(SnmpOid snmpOid) {
            return SnmpRequestTree.access$100(this.entryoids, this.entrycount, snmpOid);
        }
        
        public SnmpOid getEntryOid(int n) {
            if (this.entryoids == null) {
                return null;
            }
            if (n == -1 || n >= this.entrycount) {
                return null;
            }
            return this.entryoids[n];
        }
        
        public boolean isNewEntry(int n) {
            return this.entryoids != null && n != -1 && n < this.entrycount && this.isentrynew[n];
        }
        
        public SnmpVarBind getRowStatusVarBind(int n) {
            if (this.entryoids == null) {
                return null;
            }
            if (n == -1 || n >= this.entrycount) {
                return null;
            }
            return this.rowstatus[n];
        }
        
        public Vector<SnmpVarBind> getEntrySubList(int n) {
            if (this.entrylists == null) {
                return null;
            }
            if (n == -1 || n >= this.entrycount) {
                return null;
            }
            return this.entrylists[n];
        }
        
        public Iterator<SnmpOid> getEntryOids() {
            if (this.entryoids == null) {
                return null;
            }
            return Arrays.asList(this.entryoids).iterator();
        }
        
        public int getEntryCount() {
            if (this.entryoids == null) {
                return 0;
            }
            return this.entrycount;
        }
    }
}
